/*
# TICKET - Traffic Information Collection Kernel with Exact Timing
#
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory for the Regents of the
# University of California.
# 
# Version:	2.0.0-alpha	2003/02/20
# 
# Authors: Eric Weigle, ehw@lanl.gov or radiant-software@lanl.gov
# 
# History:
# 
#   2002-12-24 1.0.0-a  initial release
#   2003-02-20 2.0.0-a  first moderately usable release
# 
# License:
# 
# Copyright 2002. The Regents of the University of California.
# 
# This material was produced under U.S. Government contract
# W-7405-ENG-36 for Los Alamos National Laboratory, which is operated
# by the University of California for the U.S. Department of Energy.
# The Government is granted for itself and other acting on its behalf
# a paid-up, non-exclusive, irrevocable worldwide license in the
# material to reproduce, prepare derivative works, and perform
# publicly and display publicly. Beginning five (5) years after
# November 6, 2002, subject to additional five-year worldwide
# renewals, the Government is granted for itself and others acting on
# its behalf a paid-up, nonexclusive, irrevocable worldwide license
# in this material to reproduce, prepare derivative works, distribute
# copies to the public, perform publicly and display publicly, and to
# permit others to do so. NEITHER THE UNITED STATES NOR THE UNITED
# STATES DEPARTMENT OF ENERGY, NOR THE UNIVERSITY OF CALIFORNIA, NOR
# ANY OF THEIR EMPLOYEES, MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR
# ASSUMES ANY LEGAL LIABLITY OR RESPONSIBILITY FOR THE ACCURACY,
# COMPLETENESS, OR USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT,
# OR PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE
# PRIVATELY OWNED RIGHTS.
# 
# Additionally, this program is free software; you can distribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 of
# the License, or any later version. Accordingly, this program is
# distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
# License for more details. You should have received a copy of the
# GNU General Public License along with this program; if not, write
# to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
*/


// #############################################################################
// Master configuration file. You will also need to edit dump.h or route.h
//    to complete the full system configuration.
//
// These files contain all the statically-set configuration options...
// Modify with care.
// #############################################################################

#ifndef TICKET_CONFIG_H
#define TICKET_CONFIG_H

// ------------------------------------------------------------
// These hosts are used by the 'killer' program and the kernel
// code. Update to reflect the hosts on your network that will
// be referred to by the 'killer' program or the kernel code.
//
// This static mapping is required as the kernel code is
// completely passive; it won't generate or respond to 'arp'
// requests.
//
// Routing is a problem. I have ew_eth which maps physical devices and I
// have map_addrs which map logical addresses; I need to have a mapping between
// which device I'm sending over and the addresses of the source and destination;
// that is; a mapping between ew_eth and map_addrs.
//
// I do this by sorting the map_addrs table and ew_eth the same way;
// ew_eth[idx] and map_addrs[idx] talk about the same interface.  When I do
// routing for ew_eth[idx] I pick a routing partner from map_addrs[idx].partners
// and send the packet there.
// ------------------------------------------------------------
#define MODE_NONE   0x0
#define MODE_INPUT  0x1
#define MODE_OUTPUT 0x2
#define MODE_TARGET 0x4
#define HOST_ADDRESSES_TABLE \
{     /* host name,     host IP,           host MAC,           interface type, { routing targets/partners } */ \
    {     "broadcast", "255.255.255.255", "FF:FF:FF:FF:FF:FF", MODE_NONE,   {           NULL, NULL}}, /* REQUIRED. */ \
	{       "balance",     "192.168.3.1", "00:B0:D0:72:24:38", MODE_NONE,   {           NULL, NULL}}, \
	{  "balance-eth0",     "192.168.3.1", "00:B0:D0:72:24:38", MODE_NONE,   {           NULL, NULL}}, \
	{  "balance-eth1",        "10.0.1.1", "00:40:05:35:DA:84", MODE_TARGET, {"strife09-eth1", NULL}}, \
	{  "balance-eth2",        "10.0.2.1", "00:40:05:43:BD:F3", MODE_TARGET, {"strife09-eth2", NULL}}, \
	{ "strife09-eth0",   "192.168.36.90", "00:E0:81:10:EC:45", MODE_INPUT,  {           NULL, NULL}}, \
	{ "strife09-eth1",        "10.0.1.2", "00:40:05:35:DA:85", MODE_OUTPUT, { "balance-eth1", NULL}}, \
	{ "strife09-eth2",        "10.0.2.2", "00:40:05:41:94:9A", MODE_OUTPUT, { "balance-eth2", NULL}}, \
 /* {   "haste1-eth0", "192.168.37.10"  , "00:10:18:01:96:E8", MODE_INPUT|MODE_OUTPUT, {NULL,NULL} }, */ \
 /* {   "haste1-eth1", "192.168.37.11"  , "00:06:5B:8D:6E:F2", MODE_INPUT|MODE_OUTPUT, {NULL,NULL} }, */ \
 /* {   "haste2-eth0", "192.168.37.20"  , "00:06:5B:8D:97:18", MODE_INPUT|MODE_OUTPUT, {NULL,NULL} }, */ \
 /* {   "haste2-eth1", "192.168.37.21"  , "00:06:5B:8D:97:19", MODE_INPUT|MODE_OUTPUT, {NULL,NULL} }, */ \
 /* {   "haste3-eth0", "192.168.37.30"  , "00:06:5B:F2:1B:2C", MODE_INPUT|MODE_OUTPUT, {"haste1-eth0",NULL} }, */ \
 /* {   "haste3-eth1", "192.168.37.31"  , "00:06:5B:F2:1B:2D", MODE_INPUT|MODE_OUTPUT, {"haste2-eth0",NULL} }, */ \
 /* {       "swift01",   "192.168.34.10", "00:90:27:E0:08:E1", MODE_NONE,   {           NULL, NULL}}, */ \
 /* {       "swift02",   "192.168.34.20", "00:90:27:6A:F8:0A", MODE_NONE,   {           NULL, NULL}}, */ \
 /* {  "swift09-eth0",   "192.168.34.90", "00:90:27:6A:F0:E8", MODE_NONE,   {           NULL, NULL}}, */ \
 /* {  "swift09-eth1",        "10.0.3.1", "00:02:B3:9D:08:58", MODE_NONE,   {           NULL, NULL}}, */ \
 /* {   "rapid5-eth0",   "192.168.32.60", "00:A0:CC:50:E8:B0", MODE_NONE,   {           NULL, NULL}}, */ \
 /* {   "rapid6-eth0",        "10.0.2.7", "00:60:CF:20:51:A6", MODE_NONE,   {           NULL, NULL}}, */ \
 /* {   "rapid6-eth1",   "192.168.32.70", "00:60:CF:20:51:AD", MODE_NONE,   {           NULL, NULL}}, */ \
 /* {   "rapid6-eth2",        "10.0.1.7", "00:A0:CC:41:74:03", MODE_NONE,   {           NULL, NULL}}, */ \
 /* {   "rapid7-eth0",        "10.0.1.8", "00:A0:CC:40:BA:15", MODE_NONE,   {           NULL, NULL}}, */ \
 /* {   "rapid7-eth1",   "192.168.32.80", "00:60:CF:20:50:51", MODE_NONE,   {           NULL, NULL}}, */ \
 /* {   "agile1-eth1",   "192.168.33.20", "00:60:CF:20:51:B6", MODE_NONE,   {           NULL, NULL}}, */ \
 /* {   "agile1-eth1",   "192.168.33.20", "00:60:CF:20:51:B6", MODE_NONE,   {           NULL, NULL}}, */ \
 /* {   "agile3-eth1",   "192.168.33.40", "00:60:CF:20:50:97", MODE_NONE,   {           NULL, NULL}}, */ \
    {            NULL,              NULL, "00:00:00:00:00:00", MODE_NONE,   {           NULL, NULL}} /* terminator */ \
}


// ------------------------------------------------------------
// network port to put various things on. Currently only used:
//     in ../kernel/dump.c it sets port to which we send dump packets.
//     in ./user_network.c it sets port on which we listen.
//
// The kernel code, flooder, forger, and killer use this
// indirectly by referencing the two files listed above.
//
// How did I choose these numbers? Arbitrarily.
//     RADIANT = (18*1*4*9*1*14*20)%65536 = 50368
// Then I checked a very good ports list by neohapsis
//     http://www.neohapsis.com/neolabs/neo-ports/
// and ensured that nothing else commonly used this one.
// ------------------------------------------------------------
#define CLIENT_TICKET_PORT 50368
#define SERVER_TICKET_PORT 50369

// ------------------------------------------------------------
// Receive timeout, in seconds, for stuff done over UDP.
// Not really used at the moment.
// ------------------------------------------------------------
#define TICKET_RECV_TIMEOUT 5

// ------------------------------------------------------------
// Set up Kernel networking parameters.
// At 200ms and 1Gbps we will need to queue:
//     ((200/1000)(1000000000/8)/1500) =  16,666 packets.
//     At 1pkt/page we get:
//     16666*4096 =~ 65MB of memory required. Assume we have 128 for niceties.
// used in ../kernel/ticket_main.c
// TODO: where did I steal these from????
// ------------------------------------------------------------
#define NETDEV_MAX_BACKLOG 20000 /* 20000 */
#define NO_CONG_THRESH      1000 /*  1000 */
#define NO_CONG             1000 /*  1000 */
#define LO_CONG            10000 /* 10000 */
#define MOD_CONG           18000 /* 18000 */

// ------------------------------------------------------------
// Do we want to allow remote administration via raw ethernet packets?
// Define this if so; we can then reboot the machine or change states when routing.
//
// See netif_rx_* in ../kernel/network.c, ../kernel/dump.c, ../kernel/route.c
//     ../admin/admin.[ch]
// ------------------------------------------------------------
#define ALLOW_INSECURE_ADMIN

// ------------------------------------------------------------
// _default_ verbosity: each -v flag on command line increments.
//    0: critical messages (errors) only.
//    1: status reports.
//    2: debugging information.
// ------------------------------------------------------------
#define DEFAULT_VERBOSITY 1

#endif

