/*
# TICKET - Traffic Information Collection Kernel with Exact Timing
#
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory for the Regents of the
# University of California.
# 
# Version:	2.0.0-alpha	2003/02/20
# 
# Authors: Eric Weigle, ehw@lanl.gov or radiant-software@lanl.gov
# 
# History:
# 
#   2002-12-24 1.0.0-a  initial release
#   2003-02-20 2.0.0-a  first moderately usable release
# 
# License:
# 
# Copyright 2002. The Regents of the University of California.
# 
# This material was produced under U.S. Government contract
# W-7405-ENG-36 for Los Alamos National Laboratory, which is operated
# by the University of California for the U.S. Department of Energy.
# The Government is granted for itself and other acting on its behalf
# a paid-up, non-exclusive, irrevocable worldwide license in the
# material to reproduce, prepare derivative works, and perform
# publicly and display publicly. Beginning five (5) years after
# November 6, 2002, subject to additional five-year worldwide
# renewals, the Government is granted for itself and others acting on
# its behalf a paid-up, nonexclusive, irrevocable worldwide license
# in this material to reproduce, prepare derivative works, distribute
# copies to the public, perform publicly and display publicly, and to
# permit others to do so. NEITHER THE UNITED STATES NOR THE UNITED
# STATES DEPARTMENT OF ENERGY, NOR THE UNIVERSITY OF CALIFORNIA, NOR
# ANY OF THEIR EMPLOYEES, MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR
# ASSUMES ANY LEGAL LIABLITY OR RESPONSIBILITY FOR THE ACCURACY,
# COMPLETENESS, OR USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT,
# OR PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE
# PRIVATELY OWNED RIGHTS.
# 
# Additionally, this program is free software; you can distribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 of
# the License, or any later version. Accordingly, this program is
# distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
# License for more details. You should have received a copy of the
# GNU General Public License along with this program; if not, write
# to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
*/

// #############################################################################
// Kernel networking functions.
// #############################################################################
//OLD#include "ticket.h"
#include "network.h"
//  This is pretty wack, but I want to include just this data in my userspace
//  killer program too, and this is the easiest way (#include)
#include "addresses.h"
#include "config.h"

// =============================================================================
// File-global variables.
// =============================================================================
// 'global' pointers to network devices.
struct net_device *ew_eth[ETHERNET_IF_COUNT] = ETHERNET_IF_INITIALIZER;
// parallel to ew_eth; status before we muck with stuff; so we can clean up.
int ew_eth_oldstatus[ETHERNET_IF_COUNT] = ETHERNET_IF_INITIALIZER;
// parallel to ew_eth (but extended); mappings.
// passive so don't do ARP stuff.
struct name_mapping_raw map_addrs_raw[] = HOST_ADDRESSES_TABLE;
struct name_mapping     *map_addrs=NULL;
struct name_mapping *broadcast_addr;

//OLD// How many packets dropped so far?
//OLDunsigned long ticket_drops=0;

//OLD// global pointer to the receive function, too
//OLDint (*netif_rx)(struct sk_buff *skb)=&netif_rx_drop;

//OLD// =============================================================================
//OLD// Handle reception of a packet for modes where just want to drop,
//OLD// such as before I'm fully initialized.
//OLD// =============================================================================
//OLDint netif_rx_drop(struct sk_buff *skb) {
//OLD	// This code allows network reboots while in this mode.
//OLD	if (skb==NULL || skb->dev==NULL ||
//OLD		(skb->data - skb->mac.raw)!=ETH_HEADER_LEN)
//OLD		goto out;
//OLD
//OLD	skb_push(skb, ETH_HEADER_LEN);
//OLD	if (skb->len < ETH_HEADER_LEN)
//OLD		goto out;
//OLD
//OLD#ifdef ALLOW_INSECURE_ADMIN
//OLD	// (Network reboot code. C break-out logic makes it fairly low cost.)
//OLD	if (skb->len >= (ETH_HEADER_LEN+4) //+IP_HEADER_LENGTH=20
//OLD		&& skb->mac.ethernet->h_proto==0x9999
//OLD		&& skb->data[14]=='D'
//OLD		&& skb->data[15]=='I'
//OLD		&& skb->data[16]=='E'
//OLD		&& skb->data[17]=='!') {
//OLD			ew_panic("Reboot request received!");
//OLD	}
//OLD#endif
//OLD
//OLDout:
//OLD	kfree_skb(skb);
//OLD	ticket_drops++;
//OLD	return NET_RX_DROP;
//OLD}


// =============================================================================
// Sort devices and get all maps to be in parallel.
// =============================================================================
static int sort_interfaces(void) {
	struct name_mapping *addrptr;
	struct name_mapping swapspace;
	int idx, jdx;
	int kdx, ldx;

	// Print out the link information.
	for (idx=0; ew_eth[idx]!=NULL; idx++) {
		// Get address info from known table.
		if ((addrptr = get_addr_info(map_addrs, NULL, 0, ew_eth[idx]->dev_addr))) {
			// GOOD case; check to see if tables out-of-order.
			jdx = addrptr-map_addrs;
			if (idx!=jdx) {
				// DEBUG printk("Need to swap map_addrs[%d] and addrptr[0] to match with ew_eth[%d]\n", jdx, idx);
				memcpy(   &swapspace, map_addrs+jdx, sizeof(struct name_mapping));
				memcpy(map_addrs+jdx, map_addrs+idx, sizeof(struct name_mapping));
				memcpy(map_addrs+idx,    &swapspace, sizeof(struct name_mapping));
	
				// As I'm swapping, need to keep the pointers in 'partners' working.
				// Otherwise, routing output won't work later.
				// This O(n^2) could be really slow for large tables...
				for (kdx=0; map_addrs[kdx].name!=NULL; kdx++) {
					for (ldx=0; ldx<ROUTING_PARTNERS; ldx++) {
						if (map_addrs[kdx].partners[ldx]==idx) {
							map_addrs[kdx].partners[ldx]=jdx;
						} else if (map_addrs[kdx].partners[ldx]==jdx) {
							map_addrs[kdx].partners[ldx]=idx;
						}
					}
				}
			}
		} else {
			// BAD case; can't route packets, don't recognize the interface.
			char tmpbuf[32];
			printk(KERN_EMERG "Failed match of device in this machine (ew_eth[%d], '%s'):\n",
					idx, ew_eth[idx]->name);
			printk(KERN_INFO "Address: %s\n", mac2str(ew_eth[idx]->dev_addr, tmpbuf));
			printk(KERN_INFO "Table searched:\n");
			print_mapping(map_addrs);
			return 0;
		}
	}

	return 1;
}


// =============================================================================
// print out all interfaces in the local machine's table.
// Pass in a message to print, and the number of input interfaces.
// =============================================================================
int print_interfaces(char *msg, int num_input) {
	struct name_mapping *addrptr;
	int idx;

	printk(KERN_INFO "All Ethernet devices (%s):\n", msg);
	// Print out the link information.
	for (idx=0; ew_eth[idx]!=NULL; idx++) {
		if ((addrptr = get_addr_info(map_addrs, NULL, 0, ew_eth[idx]->dev_addr))) {
			// matched device, print out GOOD. Otherwise... very very bad.
			// The error case should have already been handled in the sort_devices function.
			// DEBUG printk(KERN_INFO "Device ew_eth[%d]=map_addrs[%d]=%s, ifindex=%d\n", idx, addrptr-map_addrs, ew_eth[idx]->name, ew_eth[idx]->ifindex);
			print_one_map(addrptr);
		}
	}
	return 1;
}


// =============================================================================
// Bring up all ethernet interfaces we can find.
// This does in the kernel what one would do in userland via:
//       '/sbin/ifconfig eth0 up 192.168.xx.xx netmask 0xffff0000'
//
// Save their old status so we can bring them down safely, later.
// Then sort 'em, print 'em, and return.
// =============================================================================
int ticket_ifupall(void) {
	struct net_device *netdev;
	int devnum, ethnum;

	// set up address mapping.
	map_addrs = process_mapping(map_addrs_raw);
	// DEBUG printk(KERN_INFO "All known interfaces (just after processing):\n");
	// DEBUG print_mapping(map_addrs);

	// ----------------------------------------
	// Find and open ethernet devices.
	//
	// We know that the function net_device_init in drivers/net/setup.c has executed.
	//    (via net_dev_init in net/core/dev.c);
	//    Thus, the drivers *should* be properly configured by this point.
	// ----------------------------------------
	printk(KERN_INFO "Bringing up interfaces...\n");

	rtnl_lock(); // used to be rtnl_shlock(). need to hold to use dev_get_by_index, dev_open.
	// Loop through indices to find our ethernet cards.
	// devnum 0 is invalid for some reason, start with 1. (?Why is this?)
	for (devnum=1, ethnum=0; (netdev=dev_get_by_index(devnum))!=NULL && ethnum<ETHERNET_IF_COUNT; devnum++) {
		// netdev is a valid device; what the heck is it?
		//    see include/linux/netdevice.h
		printk(KERN_INFO "    Found device '%s': %02X:%02X:%02X:%02X:%02X:%02X-- ", netdev->name,
				netdev->dev_addr[0], netdev->dev_addr[1], netdev->dev_addr[2], netdev->dev_addr[3], netdev->dev_addr[4], netdev->dev_addr[5]);

		// Is it an ethernet interface?
		// Ok, can do this more efficiently with device type field, whatever.
		if (strncmp(netdev->name, "eth", 3)==0) {
			// OK, save pointer to device for later
			ew_eth[ethnum] = netdev;

			// Finally, open it up (the actual 'up' routine)
			if (netif_running(netdev)) {
				ew_eth_oldstatus[ethnum] = NET_DEVICE_OPEN;
				printk("Already running, not opening.\n");
			} else if (dev_open(netdev)>=0) {
				ew_eth_oldstatus[ethnum] = NET_DEVICE_CLOSED;
				printk("Successfully opened.\n");
			} else {
				printk("Failed to open device %s!!!\n", netdev->name);
				printk("This probably means the device driver is unavailable; is it installed?\n");
				return 0;
			}
			ethnum++;
		} else {
			printk("Not ethernet, ignoring.\n");
			// give back the interface. Decrements use count.
			dev_put(netdev);
		}
	}
	rtnl_unlock();

	// error check.
	if (ethnum<1) {
		printk(KERN_INFO "Too few ethenet devices in system!\n");
		return 0;
	}

	// sort interfaces into order assumed later on...
	if (!sort_interfaces()) {
		printk(KERN_INFO "Failed to sort devices properly!\n");
		return 0;
	}

	// get pointer to broadcast information for later checks.
	// Have to do this _AFTER_ the sort or the pointer I get is 'pointless' :P
	if (!(broadcast_addr=get_addr_info(map_addrs, "broadcast", 0, NULL))) {
		printk(KERN_EMERG "Must have broadcast address in shared_addresses configuration table!");
		return 0;
	}

	// DEBUG // Print out the link information.
	// DEBUG if (!print_interfaces("end of ifupall()", 0)) {
	// DEBUG 	goto fail_ifupall;
	// DEBUG }
	printk(KERN_INFO "Routing table, local interfaces first:\n");
	print_mapping(map_addrs);

	//TODO // Wait 10 seconds for interfaces to come up. Do I need to do this?
	//TODO printk("Waiting for ifup...");
	//TODO for (i=10; i>0; i--) {
	//TODO 	printk("(%d)", i);
	//TODO 	mdelay(1000);
	//TODO }
	//TODO printk("Ok.\n");


	// done!
	return 1;
}


/* ==========================================================================
 * find the next output device.
 * ========================================================================== */
int next_output(struct name_mapping **output, struct name_mapping **target) {
	static int device_idx=0;
	static int partner_idx=-1;
	int device_loops;

	partner_idx++; /* initial increment; get next device */

	/* device loop; avoid infinite loops if no output devices can be located */
	for (device_loops=0; device_loops<=ETHERNET_IF_COUNT; device_loops++) {
		/* partner loop */
		for ( ; partner_idx<ROUTING_PARTNERS; partner_idx++) {
			// printk("Checking for output on %d, %d\n", device_idx, partner_idx);
			// have an output?
			if ((map_addrs[device_idx].mode&MODE_OUTPUT) &&
				 map_addrs[device_idx].partners[partner_idx]>=0) {
				*output=&(map_addrs[device_idx]);
				*target=&(map_addrs[(*output)->partners[partner_idx]]);

				return 1;
			}
		}

		/* next device logic */
		partner_idx=0;
		device_idx++;
		if (ew_eth[device_idx]==NULL) {
			device_idx=0;
		}
	}

	printk(KERN_WARNING "No output devices found (next_output)!!!\n");
	return 0;
}

