/*
# TICKET - Traffic Information Collection Kernel with Exact Timing
#
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory for the Regents of the
# University of California.
# 
# Version:	2.0.0-alpha	2003/02/20
# 
# Authors: Eric Weigle, ehw@lanl.gov or radiant-software@lanl.gov
# 
# History:
# 
#   2002-12-24 1.0.0-a  initial release
#   2003-02-20 2.0.0-a  first moderately usable release
# 
# License:
# 
# Copyright 2002. The Regents of the University of California.
# 
# This material was produced under U.S. Government contract
# W-7405-ENG-36 for Los Alamos National Laboratory, which is operated
# by the University of California for the U.S. Department of Energy.
# The Government is granted for itself and other acting on its behalf
# a paid-up, non-exclusive, irrevocable worldwide license in the
# material to reproduce, prepare derivative works, and perform
# publicly and display publicly. Beginning five (5) years after
# November 6, 2002, subject to additional five-year worldwide
# renewals, the Government is granted for itself and others acting on
# its behalf a paid-up, nonexclusive, irrevocable worldwide license
# in this material to reproduce, prepare derivative works, distribute
# copies to the public, perform publicly and display publicly, and to
# permit others to do so. NEITHER THE UNITED STATES NOR THE UNITED
# STATES DEPARTMENT OF ENERGY, NOR THE UNIVERSITY OF CALIFORNIA, NOR
# ANY OF THEIR EMPLOYEES, MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR
# ASSUMES ANY LEGAL LIABLITY OR RESPONSIBILITY FOR THE ACCURACY,
# COMPLETENESS, OR USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT,
# OR PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE
# PRIVATELY OWNED RIGHTS.
# 
# Additionally, this program is free software; you can distribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 of
# the License, or any later version. Accordingly, this program is
# distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
# License for more details. You should have received a copy of the
# GNU General Public License along with this program; if not, write
# to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
*/

// #############################################################################
// Kernel networking header
// #############################################################################

#ifndef EW_TICKET_COMMON_NETWORK_H
#define EW_TICKET_COMMON_NETWORK_H

/* Standard headers for kernel modules */
#include <linux/modversions.h> 
#include <linux/module.h>
#include <linux/tty.h>

/* other headers */
#include <linux/netdevice.h>  /* network device structure, netif_rx */
#include <linux/in.h>         /* sockaddr_in, IPPROTO_UDP */
#include <linux/skbuff.h>     /* socket buffers (struct skb) */
#include <linux/udp.h>        /* struct udphdr */
#include <linux/ip.h>         /* struct iphdr */
#include <asm/uaccess.h>      /* required for checksum.h to compile cleanly; VERIFY_WRITE */
#include <linux/in6.h>        /* required for checksum.h to compile cleanly; struct in6_addr */
#include <asm/checksum.h>     /* ip_fast_csum */
#include <linux/rtnetlink.h>  /* for rtnl_(un)lock(), must hold to access device tables */
#include <linux/delay.h>      /* mdelay */
#include "addresses.h" /* for name_mapping */
//OLD#include <linux/inetdevice.h> /* for devinet_ioctl() */
//OLD#include <linux/if_ether.h>   /* Defines ETH_ALEN, ETH_HLEN, ETH_DATA_LEN, ETH_FRAME_LEN, and struct ethhdr. */
//OLD#include <linux/tcp.h>        /* Defines struct tcphdr */
//OLD#include "./common/shared_network.h"

/* these are stolen from linux/byteorder/swab.h
 * Need to figure out how to get htons ntohs to resolve on module load
 * But they're inlines, right???
 * TODO; make this portable. This is little-endian specific.
 * 	Yeah, like all the other kernel code I wrote _isn't_. Ha... Ha... Ha.
 */
#define ehw_htons(x) \
({ \
	__u16 __x = (x); \
	((__u16)( \
		(((__u16)(__x) & (__u16)0x00ffU) << 8) | \
		(((__u16)(__x) & (__u16)0xff00U) >> 8) )); \
})

#define ehw_ntohs(x) \
({ \
	__u16 __x = (x); \
	((__u16)( \
		(((__u16)(__x) & (__u16)0x00ffU) << 8) | \
		(((__u16)(__x) & (__u16)0xff00U) >> 8) )); \
})


// Ethernet interfaces in this box.
#define ETHERNET_IF_COUNT 8
#define ETHERNET_IF_INITIALIZER {0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L}
extern struct net_device *ew_eth[ETHERNET_IF_COUNT];
#define NET_DEVICE_CLOSED 0x0
#define NET_DEVICE_OPEN   0x1
extern int ew_eth_oldstatus[ETHERNET_IF_COUNT];

/* OLD STUFF
	// How many packets have been dropped.
	extern unsigned long ticket_drops;

	// =============================================================================
	// STRUCTURES
	// =============================================================================

	// These are stored in NBO
	// Makes passing parameters around easier.
	struct address_pair {
		unsigned char mac_addr[ETH_ALEN];
		unsigned int ip_addr;
	};
	struct address_pairs {
		unsigned char dst_hw[ETH_ALEN];
		unsigned char src_hw[ETH_ALEN];
		unsigned int dst_ip;
		unsigned int src_ip;
	};
*/

// =============================================================================
// FUNCTIONS
// =============================================================================
// print out all interfaces in the local machine's table.
int print_interfaces(char *msg, int num_input);
// Bring up all ethernet interfaces we can find.
int ticket_ifupall(void);
// The next output device.
int next_output(struct name_mapping **output, struct name_mapping **target);

#endif

