/*
# TICKET - Traffic Information Collection Kernel with Exact Timing
#
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory for the Regents of the
# University of California.
# 
# Version:	2.0.0-alpha	2003/02/20
# 
# Authors: Eric Weigle, ehw@lanl.gov or radiant-software@lanl.gov
# 
# History:
# 
#   2002-12-24 1.0.0-a  initial release
#   2003-02-20 2.0.0-a  first moderately usable release
# 
# License:
# 
# Copyright 2002. The Regents of the University of California.
# 
# This material was produced under U.S. Government contract
# W-7405-ENG-36 for Los Alamos National Laboratory, which is operated
# by the University of California for the U.S. Department of Energy.
# The Government is granted for itself and other acting on its behalf
# a paid-up, non-exclusive, irrevocable worldwide license in the
# material to reproduce, prepare derivative works, and perform
# publicly and display publicly. Beginning five (5) years after
# November 6, 2002, subject to additional five-year worldwide
# renewals, the Government is granted for itself and others acting on
# its behalf a paid-up, nonexclusive, irrevocable worldwide license
# in this material to reproduce, prepare derivative works, distribute
# copies to the public, perform publicly and display publicly, and to
# permit others to do so. NEITHER THE UNITED STATES NOR THE UNITED
# STATES DEPARTMENT OF ENERGY, NOR THE UNIVERSITY OF CALIFORNIA, NOR
# ANY OF THEIR EMPLOYEES, MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR
# ASSUMES ANY LEGAL LIABLITY OR RESPONSIBILITY FOR THE ACCURACY,
# COMPLETENESS, OR USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT,
# OR PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE
# PRIVATELY OWNED RIGHTS.
# 
# Additionally, this program is free software; you can distribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 of
# the License, or any later version. Accordingly, this program is
# distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
# License for more details. You should have received a copy of the
# GNU General Public License along with this program; if not, write
# to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
*/

// #############################################################################
// Dump header information, shared between user and kernel programs:
//    kernel/dump.c, user{reader, writer, printer}
//
// Gives info on collected information, record sizes, etc.
// #############################################################################

#ifndef TICKET_COMMON_ROUTE
#define TICKET_COMMON_ROUTE

// This is a table defining a state machine.
// Each line is a state (mode) with certain a certain delay & drop configuration.
//
//   idx     : Index, must be the actual C index into the array.
//   nextidx : Index of next state to which to move, when `duration' is over.
//   duration: How long to stay in this state:
//             3,600,000 is an hour (1000*60*60)
//             can't be longer than 2^32/1000 = 4,294,967ms = 4,294s = 71m.
//                 if you need it longer, put in duplicate states.
//   delay   : Delay in milliseconds before forwarding packets.
//   drop    : Drop every Nth packet (0=nothing, 1=everything)
//
// This is a SLOPPY delay mechanism; when the delay falls, any packets that are
//     queued use the new delay. That is, if several packets are queued with a
//     100ms delay and then the delay falls to 10ms, if packets are still in the
//     queue they will experience a delay between 10 and 100ms.
//
// Packet reordering *CANNOT* occur in the current implementation. So if you go
//     to a lower delay the first couple of packets will be delayed an arbitrary
//     value between the lower and higher delay.
//
// Watch your 'next_delay' indices, if they don't loop somewhere, you core dump.
//
// see ../kernel/route.c
//
// TODO: cache time of last administrative attempt; if get new one within a certain window, ignore it.
//       I put two identical entries here because I administer by broadcast;
//       two interfaces means two copies of the message means need two entries
//       for a single effect per administration attempt. (otherwise skip one).
#define ROUTING_CONFIG \
  /* idx, delay, duration, drop, nextidx */ \
	{  0,    50, 3600000,    0,       1 }, \
	{  1,    25, 3600000,    0,       2 }, \
	{  2,    25, 3600000,    0,       3 }, \
	{  3,    25, 3600000,    0,       4 }, \
	{  4,    50, 3600000,    0,       5 }, \
	{  5,    50, 3600000,    0,       6 }, \
	{  6,   100, 3600000,    0,       7 }, \
	{  7,   100, 3600000,    0,       0 }

/* maximum number of packets to allow in queue before dropping them */
/* 300 is what netdev_max_backlog is... seems reasonable  */
#define TICKET_ROUTE_BACKLOG 500

#endif
