/*
# TICKET - Traffic Information Collection Kernel with Exact Timing
#
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory for the Regents of the
# University of California.
# 
# Version:	2.0.0-alpha	2003/02/20
# 
# Authors: Eric Weigle, ehw@lanl.gov or radiant-software@lanl.gov
# 
# History:
# 
#   2002-12-24 1.0.0-a  initial release
#   2003-02-20 2.0.0-a  first moderately usable release
# 
# License:
# 
# Copyright 2002. The Regents of the University of California.
# 
# This material was produced under U.S. Government contract
# W-7405-ENG-36 for Los Alamos National Laboratory, which is operated
# by the University of California for the U.S. Department of Energy.
# The Government is granted for itself and other acting on its behalf
# a paid-up, non-exclusive, irrevocable worldwide license in the
# material to reproduce, prepare derivative works, and perform
# publicly and display publicly. Beginning five (5) years after
# November 6, 2002, subject to additional five-year worldwide
# renewals, the Government is granted for itself and others acting on
# its behalf a paid-up, nonexclusive, irrevocable worldwide license
# in this material to reproduce, prepare derivative works, distribute
# copies to the public, perform publicly and display publicly, and to
# permit others to do so. NEITHER THE UNITED STATES NOR THE UNITED
# STATES DEPARTMENT OF ENERGY, NOR THE UNIVERSITY OF CALIFORNIA, NOR
# ANY OF THEIR EMPLOYEES, MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR
# ASSUMES ANY LEGAL LIABLITY OR RESPONSIBILITY FOR THE ACCURACY,
# COMPLETENESS, OR USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT,
# OR PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE
# PRIVATELY OWNED RIGHTS.
# 
# Additionally, this program is free software; you can distribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 of
# the License, or any later version. Accordingly, this program is
# distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
# License for more details. You should have received a copy of the
# GNU General Public License along with this program; if not, write
# to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
*/

// #############################################################################
// TICKET main file; basically just switches to proper mode file.
//
// From Linux Kernel Internals, (www.moses.uklinux.net/patches/lki-2.html#ss2.2)
//   "Running at processor's ring 0 (on x86, that is) implies that the
//   kernel threads enjoy all I/O privileges and cannot be pre-empted by the
//   scheduler."
//
// Uhm... Uhm... a timer tick occurs, and the interrupt happens, Period.
// schedule() can be called at that point, right? I'm dumb...
//
// #############################################################################
#include "ticket.h"
#include "network.h"

#include <linux/reboot.h>
#include <linux/sched.h> /* for daemonize */

// =============================================================================
// Function declarations
// =============================================================================
// A couple of function declarations from the dump code.
void dump_init(char **args);
int netif_rx_dump(struct sk_buff *skb);
// and the same ones from the routing code.
void route_init(char **args);
int netif_rx_route(struct sk_buff *skb);
extern int (*netif_rx)(struct sk_buff *skb);

// =============================================================================
// File-global variables.
// =============================================================================
// How many packets I _know_ I've lost. May lose packets w/o knowing though!
// (as when the NIC drops them and doesn't tell me)
extern unsigned long ticket_drops;
// The list of packets we're working with
struct sk_buff_head ticket_skbs;
// Parameters used to control behavior
extern int netdev_max_backlog;
extern int no_cong_thresh;
extern int no_cong;
extern int lo_cong;
extern int mod_cong;

// =============================================================================
// The main routine, called by the actual init() function.
//
// Just daemonizes, parses command-line parameters,
// brings up interfaces, and calls appropriate handler
// =============================================================================
void ticket_init(char **argv_init, char **envp_init) {
	printk("\nTICKET code running.\n");

	// explicitly get rid of any user level claims
	daemonize();

	// This is annoying, but we have to set up this state before
	// bringing up the interfaces; otherwise there's a race-- if
	// a packet arrives after the interface is up but before the
	// state is set, we'll kernel panic. Thus we do it here.
	skb_queue_head_init(&ticket_skbs);
	ticket_drops=0;

	// ----------------------------------------
	// Bring up interfaces
	// ----------------------------------------
	printk("Bringing up interfaces...\n");
	if (!ifupall()) {
		ew_panic("Interface initialization failed. D'oh!");
	}

	// Kernel uses some parameters used to control network behavior
	// We want to muck with them in all modes.
	netdev_max_backlog=NETDEV_MAX_BACKLOG;
	no_cong_thresh=NO_CONG_THRESH;
	no_cong=NO_CONG;
	lo_cong=LO_CONG;
	mod_cong=MOD_CONG;

	// ----------------------------------------
	// Begin actual work. Do not return.
	// ----------------------------------------
	if (TICKET_MODE == TICKET_MODE_DUMP) {
		netif_rx = &netif_rx_dump;
		dump_init(envp_init);
	} else if (TICKET_MODE == TICKET_MODE_ROUTE) {
		netif_rx = &netif_rx_route;
		route_init(envp_init);
	} else {
		ew_panic("Bad mode parameter!");
	}

	ew_panic("ERRONEOUSLY RETURNED FROM A NONRETURNABLE FUNCTION!");
}


// =============================================================================
// Something really bad has happened; tell user and then reboot.
// =============================================================================
void ew_panic(const char *msg) {
	int reboot_delay;

	printk(KERN_EMERG "THE HORROR! '%s' has occurred.\n", msg);

	reboot_delay = 10;
	printk(KERN_EMERG "Rebooting in %d seconds... ", reboot_delay);
	while (reboot_delay>0) {
		printk("%d.", reboot_delay);
		mdelay(1000);
		reboot_delay--;
	}
	printk("NOW!\n\n\n");

	machine_restart(NULL);
}

