/*
# TICKET - Traffic Information Collection Kernel with Exact Timing
#
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory for the Regents of the
# University of California.
# 
# Version:	2.0.0-alpha	2003/02/20
# 
# Authors: Eric Weigle, ehw@lanl.gov or radiant-software@lanl.gov
# 
# History:
# 
#   2002-12-24 1.0.0-a  initial release
#   2003-02-20 2.0.0-a  first moderately usable release
# 
# License:
# 
# Copyright 2002. The Regents of the University of California.
# 
# This material was produced under U.S. Government contract
# W-7405-ENG-36 for Los Alamos National Laboratory, which is operated
# by the University of California for the U.S. Department of Energy.
# The Government is granted for itself and other acting on its behalf
# a paid-up, non-exclusive, irrevocable worldwide license in the
# material to reproduce, prepare derivative works, and perform
# publicly and display publicly. Beginning five (5) years after
# November 6, 2002, subject to additional five-year worldwide
# renewals, the Government is granted for itself and others acting on
# its behalf a paid-up, nonexclusive, irrevocable worldwide license
# in this material to reproduce, prepare derivative works, distribute
# copies to the public, perform publicly and display publicly, and to
# permit others to do so. NEITHER THE UNITED STATES NOR THE UNITED
# STATES DEPARTMENT OF ENERGY, NOR THE UNIVERSITY OF CALIFORNIA, NOR
# ANY OF THEIR EMPLOYEES, MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR
# ASSUMES ANY LEGAL LIABLITY OR RESPONSIBILITY FOR THE ACCURACY,
# COMPLETENESS, OR USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT,
# OR PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE
# PRIVATELY OWNED RIGHTS.
# 
# Additionally, this program is free software; you can distribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 of
# the License, or any later version. Accordingly, this program is
# distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
# License for more details. You should have received a copy of the
# GNU General Public License along with this program; if not, write
# to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
*/

/*
#############################################################################
TICKET module ``dump'' code, main file.
functions for
    packet allocation,
    packet reception,
    header collection,
    packet transmission,
etc.
-----------------------------------------------------------------------------
Notes:
	indent -kr -i8
	/usr/src/linux/include/linux/init.h
	/usr/src/linux/include/linux/module.h
#############################################################################
*/

/* Standard headers for kernel modules */
#include <linux/modversions.h>
#include <linux/module.h>
#include <linux/tty.h>

/* network headers, netif_rx */
#include <linux/netdevice.h>
/* for rtnl_shlock(), etc (need to hold to use dev_get_by_index, dev_open) */
#include <linux/rtnetlink.h>

/* user space stuff */
#include <asm/uaccess.h>

//OLD#include "ticket.h"
#include "../common/network.h"
#include "../common/config.h"
#include "../common/kernel_user.h"
#include "../common/addresses.h"
#include "../common/dump.h"
#include "../common/kthread.h"
#include <asm/smplock.h>     /* (un)*lock_kernel */
#include <linux/tcp.h> /* TCP headers for collection */
//OLD#include <asm/atomic.h>

/* ================================================================================
 * Globals
 * ============================================================================= */
struct ew_thread packet_thread; /* ALL thread data */
int pass_through=1; /* DO pass stuff to netif_rx after I see it */
unsigned long ticket_drops=0;
/* Possible bug on module onload: say module A messes with netif_rx pointer;
 * then I load and copy that pointer. Later B unloads, then I unload.
 * I reset the pointer to B's netif_rx function--> core dump! */
int (*old_netif_rx)(struct sk_buff *skb);
#if LINUX_VERSION_CODE>=0x020420
int (*old_netif_receive)(struct sk_buff *skb);
#endif
extern struct name_mapping *map_addrs; /* from network.c */
// The current packet to which we are dumping.
//   Use skb->cb[0..7] for timestamp
//          64 bit cycle counters
//       skb->cb[8..11] for collection index
//		    (the place we're at in the packet (0..DUMPS_PER_PACKET-1))
// Has to be global so we can free it outside of everything.
struct sk_buff *output_skb=NULL;

/* ================================================================================
 * Function Declarations
 * ============================================================================= */
//OLDint ticket_ifupall(void);
int ticket_dump_init(void);
void ticket_dump_exit(void);
int netif_rex_dump(struct sk_buff *skb, int (*pass_through_function)(struct sk_buff *skb));

// The list of packets we're working with
struct sk_buff_head ticket_skbs;

// =============================================================================
// Allocate a packet and reserve space for the headers.
// =============================================================================
static struct sk_buff *
alloc_packet(void) {
	struct sk_buff *skb;

	// ----------------------------------------
	// INITIALIZE SKB
	// ----------------------------------------
	// Allocate skb; We should have excess room to allow for alignment.
	// See header sizes in dump_common.h, it works out (?)
	if ((skb = alloc_skb(ETH_HEADER_LEN+ETH_DATA_LEN, GFP_ATOMIC))==NULL) {
		return NULL;
	}

	// ----------------------------------------
	// RESERVE SPACE
	// ----------------------------------------
	skb_reserve(skb, ETH_HEADER_LEN+2); // reserve ethernet space (14+2=16, upper-aligned)
	skb_reserve(skb, IP_HEADER_LEN);    // reserve IP space, 20-byte IP header
	skb_reserve(skb, UDP_HEADER_LEN);   // reserve UDP space, 8 byte UDP header.
	skb_reserve(skb, DUMP_HEADER_LEN);  // reserve 8 bytes for sequence number of 'upper' header.

	// lay claim to the rest of the packet as our data area.
	(void)skb_put(skb, DUMP_RECORD_LEN*DUMPS_PER_PACKET);
	// zeroing it out _may_ not be required?
	memset(skb->data, 0, DUMP_RECORD_LEN*DUMPS_PER_PACKET);
	// skb->cb[0..7] for timestamp (64 bit/8 byte cycle counters)
	// skb->cb[8..11] for collection index (32 bit/4 byte int); (0..DUMPS_PER_PACKET-1))
	memset(skb->cb, 0, 12); 

	// done.
	return skb;
}


// =============================================================================
// Fill in the headers, top down, that we reserved before.
// =============================================================================
static int
fill_headers(struct sk_buff *skb, unsigned long long seqno) {
	struct name_mapping *output_map, *target_map;

	// Get stripe output.
	if (!next_output(&output_map, &target_map)) {
		printk("Failed to find _any_ output device!\n");
		printk("This is a configuration error.\n");
		return 0;
	}

	// have the output device.
	// DEBUG printk("Sending output via device %s to target %s\n", output_map->name, target_map->name);

	// put device into skb (recall map_addrs and ew_eth sorted identically; get same idx'd device.
	skb->dev = ew_eth[output_map-map_addrs];

	// ----------------------------------------
	// My simple header. Just the sequence no.
	// ----------------------------------------
	memcpy(skb_push(skb, DUMP_HEADER_LEN), &seqno, DUMP_HEADER_LEN);

	// ----------------------------------------
	// UDP header.
	// ----------------------------------------
	// UDP/TCP use parts of the IP header in their checksumming, hence the weird call series here.
	skb->h.uh = (struct udphdr *)skb_push(skb, UDP_HEADER_LEN);
	skb->h.uh->source = ehw_htons(CLIENT_TICKET_PORT);
	skb->h.uh->dest = ehw_htons(SERVER_TICKET_PORT);
	skb->h.uh->len = ehw_htons((DUMPS_PER_PACKET*DUMP_RECORD_LEN)+DUMP_HEADER_LEN+UDP_HEADER_LEN); // udp length includes header.

	// ARGH!!!!! Apparently, nobody really uses this checksum in ipv4...
	skb->h.uh->check=0;

	// ----------------------------------------
	// IP header.
	// ----------------------------------------
	skb->nh.iph = (struct iphdr *)skb_push(skb, IP_HEADER_LEN);
	skb->nh.iph->version = 4;
	skb->nh.iph->ihl = IP_HEADER_LEN>>2; // bytes --> 32-bit words
	skb->nh.iph->tos = 0; // change to one of the 'low latency' etc things?
	skb->nh.iph->tot_len = ehw_htons(ehw_ntohs(skb->h.uh->len)+IP_HEADER_LEN);
	skb->nh.iph->id = ehw_htons(seqno&0xffff); // kifes lowest 16 bits
	skb->nh.iph->frag_off = ehw_htons(0);
	skb->nh.iph->ttl = 64; // Arbitrary.
	skb->nh.iph->protocol = IPPROTO_UDP;
	skb->nh.iph->saddr = output_map->ip; // already in network byte order.
	skb->nh.iph->daddr = target_map->ip; // already in network byte order.

	// By comparison, IP checksum is a cinch.
	skb->nh.iph->check = 0;
	skb->nh.iph->check = ip_fast_csum((unsigned char *)skb->nh.iph, skb->nh.iph->ihl);

	// ----------------------------------------
	// Ethernet header.
	// ----------------------------------------
	skb->mac.ethernet = (struct ethhdr *)skb_push(skb, ETH_HEADER_LEN);
	memcpy(skb->mac.ethernet->h_source, output_map->mac, skb->dev->addr_len);
	memcpy(skb->mac.ethernet->h_dest,   target_map->mac, skb->dev->addr_len);
	skb->mac.ethernet->h_proto = ehw_htons(0x800); // IP TOS in NBO
	return 1;
}


// =============================================================================
// The actual function to collect information from the skbuff
// and save it to our headers structure.
//
// TICKET  takes just specific pieces of information;
// TCPDUMP takes all of the first OUT_SNAPLEN bytes.
// =============================================================================
#ifdef DUMP_MODE_TICKET
void collect_data(struct sk_buff *skb, struct collected_headers_ticket *hdr) {
	// We want the headers back so we can look at 'em.
	skb_push(skb, ETH_HEADER_LEN);

	// first, copy length
	hdr->length = skb->len;

	// Then parse out information.
	if (skb->len >= ETH_HEADER_LEN) {
		// grab timestamp out of cb (control buffer)
		memcpy(&(hdr->timestamp), &(skb->cb[0]), 8);

		// Set up pointers
		skb->mac.ethernet = (struct ethhdr *)skb->data;

		// Grab ethernet header.
		memcpy(&(hdr->mac_dst[0]), &(skb->mac.ethernet->h_dest[0]), ETH_ALEN);
		memcpy(&(hdr->mac_src[0]), &(skb->mac.ethernet->h_source[0]), ETH_ALEN);
		hdr->toslen = skb->mac.ethernet->h_proto;

		// Is it an IP packet? (0x800 NBO is 0x8 HBO due to endian flip!)
		if (ehw_ntohs(hdr->toslen) == 0x800 && skb->len >= IP_HEADER_LEN) {
			// Set up pointers
			skb_pull(skb, ETH_HEADER_LEN);
			skb->nh.iph = (struct iphdr *)(skb->data);

			// Grab IP header.
			hdr->ip_src     = skb->nh.iph->saddr;
			hdr->ip_dst     = skb->nh.iph->daddr;
			hdr->ip_len     = skb->nh.iph->tot_len;
			hdr->ip_hdr_len = (skb->nh.iph->ihl<<2); // do conversion words-->bytes
			hdr->protocol   = skb->nh.iph->protocol;

			// Grab TCP or UDP headers.
			// 20020912- oops, forgot to verify that it's not an IP frag! See RFC 791.
			// frag_off is 16 bits; 3 bits of flag (reserved, don't fragment, last fragment)
			//                      13 bits of offset.
			// #1#X XXXX XXXX XXXX --> 0x1fff
			// DEBUG printk("Packet ID: %u, Frag off: %04X\n", skb->nh.iph->id, skb->nh.iph->frag_off);
			if (skb->len >= hdr->ip_hdr_len && (ehw_ntohs(skb->nh.iph->frag_off)&0x1fff)==0) {
				// Set up pointers
				skb_pull(skb, hdr->ip_hdr_len);

				// Is it a TCP or UDP packet? (no ntoh required; it's a byte)
				if (hdr->protocol==6 && skb->len >= TCP_HEADER_LEN) {
					// Set up pointers
					skb->h.th = (struct tcphdr *)(skb->data);
					// copy data
					hdr->src_port = skb->h.th->source;
					hdr->dst_port = skb->h.th->dest;
					hdr->upper_len.tcp_len.len = skb->h.th->doff; // `data offset'
					hdr->upper_len.tcp_len.pad = '\0'; // Now explicitly nulled (after first data set collected!)
				} else if (hdr->protocol==17 && skb->len >= UDP_HEADER_LEN) {
					// Set up pointers
					skb->h.uh = (struct udphdr *)(skb->data);
					// copy data
					hdr->src_port = skb->h.uh->source;
					hdr->dst_port = skb->h.uh->dest;
					hdr->upper_len.udp_len = skb->h.uh->len;
				} /* TCP or UDP packet */
			} /* first-frag IP packet? */
		} /* IP packet? */
	} /* valid upper-level data? */
}
#endif
#ifdef DUMP_MODE_TCPDUMP
void collect_data(struct sk_buff *skb, struct collected_headers_tcpdump *hdr) {
	// timestamp.
	memcpy(&(hdr->timestamp), &(skb->cb[0]), sizeof(unsigned long long));

	// save length.
	hdr->pktlen = skb->len;

	// data. (handling short packets)
	memcpy(hdr->snap, skb->data, skb->len < OUT_SNAPLEN ? skb->len : OUT_SNAPLEN);
}
#endif

int netif_rx_dump(struct sk_buff *skb) {
	return netif_rex_dump(skb, old_netif_rx);
}
#if LINUX_VERSION_CODE>=0x020420
int netif_receive_dump(struct sk_buff *skb) {
	return(netif_rex_dump(skb, old_netif_receive));
}
#endif

/* -----------------------------------------------------------------------------
 * netif_rx that will do appropriate TICKET ops (collecting header data, dumping).
 * -------------------------------------------------------------------------- */
int netif_rex_dump(struct sk_buff *skb, int (*pass_through_function)(struct sk_buff *skb)) {
	register unsigned int dummylo;
	register unsigned int dummyhi;
	struct sk_buff *copy;

	// Get the timestamp into the cb area.
	// Chose to inline versus calling get_timestamp for efficiency.
	// (no function call overhead).
	asm volatile (".byte 0x0f, 0xa2" : : : "eax", "ebx", "ecx", "edx");
	asm volatile (".byte 0x0f, 0x31;              # the RDTSC instruction
					movl %%eax, %0
					movl %%edx, %1"
					: "=r" (dummylo), "=r" (dummyhi)
					:
					: "eax", "edx"
					);
	*((unsigned long long *)skb->cb) = ((unsigned long long)dummyhi)<<32 | dummylo;

	// First, check the packet. It should have the eth. headers removed,
	// but still valid (in the packet)
	if ((skb->data - skb->mac.raw)==ETH_HEADER_LEN) {
		// if passing through, get a copy!
		if (pass_through) {
			copy = skb_copy(skb, GFP_ATOMIC);
		} else {
			copy = skb;
			skb = NULL;
		}

		if (copy) {
#ifdef ALLOW_INSECURE_ADMIN
			// (Network reboot code. C break-out logic makes it fairly low cost.)
			if (copy->len >= (ETH_HEADER_LEN+4) && copy->mac.ethernet->h_proto==0x9999) {
				if (memcmp(&(copy->data[ETH_HEADER_LEN]), "DIE!", 4)==0) {
					printk(KERN_EMERG "Reboot request received, but not configured yet!\n");
					machine_restart(NULL); // TODO: check
				} else {
					printk(KERN_WARNING "Unknown administration attempt.\n");
				}
				kfree_skb(skb);
				return NET_RX_DROP;
			}
#endif

			// Enqueue the packet for later.
			// Gets spinlock, so is 'safe'
			skb_queue_tail(&ticket_skbs, copy);

			// notify init that there's a packet.
			up(&packet_thread.packet_wait_sem);
		} else {
			printk("Can't copy my skb!");
			ticket_drops++;
		}
	} else {
		// shouldn't get here; Packet failed ethernet check!
		printk(KERN_WARNING "Nonforwardable packet recieved.");
		// DEBUG	bindump(skb->data, skb->len);
		ticket_drops++;
	}

	// maybe Linux can do something with it, though...
	if (skb) {
		if (pass_through) {
			return(pass_through_function(skb));
		} else {
			kfree_skb(skb);
		}
	}
	return NET_RX_DROP;
}


// =============================================================================
// The second-level 'main' function of the collection code.
//
// Called by ticket_dump_init() (or module load code; it's the thread function).
// TODO: does it still work if called from ticket_dump_init???
//
// This code is essentially the bottom half of the packet reception code:
//   netif_rx
//      get & queue packet, up()s semaphore this code sleeps upon.
//   The loop below then
//      allocates a collection packet (alloc_packet()),
//      collects headers into it (collect_data()),
//      fills in the destination headers (fill_headers()),
//      sends it off (hard_start_xmit()).
// =============================================================================
void ticket_dump_thread(void) {
	// The current packet whose data is being collected.
	struct sk_buff *input_skb=NULL;
	// We use a sequence number to check reliability.
	unsigned long long seqno=0;
	// and a pointer to make lower code more readable.
	int *idx=NULL;
	struct name_mapping *input_map;

	lock_kernel(); /* set up thread environment; get BKL to modify low-level process stuff. */
	packet_thread.thread = current; /* fill in thread structure */
	sprintf(current->comm, "ticket_dump"); /* set name of this process; sizeof(comm)==16 */
	unlock_kernel(); /* let others run */
	up(&packet_thread.control_sem); /* Completed initialization; let creator finish */

	/* Must grab this late in the game; don't want to muck with control
	 * flow by getting packets in interrupt context for thread we're starting. */
	netif_rx = &(netif_rx_dump);
#if LINUX_VERSION_CODE>=0x020420
	netif_receive_skb = &netif_receive_dump;
#endif

#ifdef DUMP_MODE_TICKET
	printk(KERN_INFO "TICKET-style kernel dump thread running; stole netif_rx.\n");
#endif
#ifdef DUMP_MODE_TCPDUMP
	printk(KERN_INFO "TCPDUMP-style kernel dump thread running; stole netif_rx.\n");
#endif

	// Set all input devices promiscuous.
	for (input_map=map_addrs; ew_eth[input_map-map_addrs]!=NULL; input_map++) {
		if (input_map->mode & MODE_INPUT) {
			dev_set_promiscuity(ew_eth[input_map-map_addrs], 1);
		}
	}

	// Loop "forever" (break if packet_thread.die, later)
	while (1) {
		// Put myself to sleep; when wakes, there's a packet to handle.
		down(&packet_thread.packet_wait_sem);

		/* Make sure flags are visible on all CPUs. */
		/* EWEW??? Apparently this can be pretty darn painful (expensive). */
		mb();

		/* Are we back to die, or to process a packet? */
		if (packet_thread.die) {
			break;
		}

		/* this is normal work to do */
		// DEBUG printk("Got a packet in ticket_dump_init\n");

		// Get the packet off the list.
		if ((input_skb = skb_dequeue(&ticket_skbs))==NULL) {
			printk("Dequeued NULL skb in init!!!\n");
			continue; // shouldn't happen, but don't die.
		}

		// ------------------------------------------------------------
		// Which device did it come in on?
		// TODO: this is kind of slow... Worth it?
		// ------------------------------------------------------------
		input_map = map_addrs;
		while (ew_eth[input_map-map_addrs]!=input_skb->dev && ew_eth[input_map-map_addrs]) {
			 input_map++;
		}
		if (ew_eth[input_map-map_addrs]!=input_skb->dev) {
			// DEBUG printk("Failed to look up input device %s\n", input_skb->dev->name);
			// This shouldn't happen.
			goto ticket_dump_end;
		}
		// DEBUG printk("Received input on %s (%s)\n", input_map->name, input_skb->dev->name);
		if (!(input_map->mode & MODE_INPUT)) {
			// DEBUG printk("Ignoring a packet on a non-input interface.\n");
			goto ticket_dump_end;
		}

		// ------------------------------------------------------------
		// Set up output packet (allocate if required, get pointers).
		// ------------------------------------------------------------
		if (output_skb==NULL) {
			output_skb=alloc_packet();
			if (output_skb==NULL) {
				printk(KERN_EMERG "Failed to allocate a dump packet!");
				goto ticket_dump_end;
			}
		}
		// Get easier-to-use pointer of current dump header index.
		idx = (int *)(&(output_skb->cb[8]));

		// ------------------------------------------------------------
		// Collect dump data into the my_index'th field of the skb.
		// ------------------------------------------------------------
		collect_data(input_skb, &(((collected_headers *)output_skb->data)[(*idx)++]));

		// -------------------------------------------------
		// IF packet is full, send it off.
		// -------------------------------------------------
		if (*idx >= DUMPS_PER_PACKET) {
			// fill in headers of packet
			if (fill_headers(output_skb, seqno++)) {
				// force the packet to be sent *NOW*
				//   the transmission code frees the skb.
				output_skb->dev->hard_start_xmit(output_skb, output_skb->dev);
			} else {
				printk("Throwing away a perfectly good SKB.\n");
				kfree_skb(output_skb);
			}
			// This will make us allocate a new one on next loop.
			output_skb=NULL;
		}

ticket_dump_end:
		// --------------------------------------------------
		// Finish up, drop the packet we just processed.
		// --------------------------------------------------
		kfree_skb(input_skb);
	}

	/* cleanup the thread, quit. */
	lock_kernel(); /* get BKL; will muck with junk */
	packet_thread.thread = NULL; /* Thread is to-be gone */
	mb(); /* propogate flags */
	up(&packet_thread.control_sem); /* let cleanup_module know. */
	/* In theory; the kernel_thread that called clone() now calls do_exit() here,
	 * which releases BKL for me. */
}


/* -----------------------------------------------------------------------------
 * start up module. This function:
 *     steals netif_rx,
 *     brings up all interfaces to use,
 *     starts dump thread.
 * -------------------------------------------------------------------------- */
int ticket_dump_init(void) {
	printk(KERN_INFO "Loading TICKET dump module...\n");
	printk("    %d:1 Packets in:out ratio.\n", DUMPS_PER_PACKET);
	printk("    %d bytes collected per input packet.\n", DUMP_RECORD_LEN);
	printk("    %d bytes available per output packet.\n", LEFTOVER_SPACE);

	old_netif_rx = netif_rx; /* set first so failures can just call ticket_exit */
#if LINUX_VERSION_CODE>=0x020420
	old_netif_receive = netif_receive_skb_orig;
#endif
	
	/* bring up all interfaces */
	if (!ticket_ifupall()) {
		ticket_dump_exit();
		/* failure */
		return 1;
	}

	/* create new kernel thread. Zero everything first */
	memset(&packet_thread, '\0', sizeof(struct ew_thread));
	/* set up packet queue too */
	skb_queue_head_init(&ticket_skbs);

	/* Initialize everything as locked; next down will block. */
	init_MUTEX_LOCKED(&packet_thread.control_sem); /* control creation/destruction of thread */
	init_MUTEX_LOCKED(&packet_thread.packet_wait_sem); /* thread sleeps here waiting for packets */

	/* create the new thread by running a task through keventd */
	/* initialize the task queue structure */
	INIT_LIST_HEAD(&packet_thread.taskq.list);
	packet_thread.function = &ticket_dump_thread;
	packet_thread.taskq.routine = ew_thread_create;
	packet_thread.taskq.data = &packet_thread;

	/* schedule its execution, then let it run 'till init is complete... */
	schedule_task(&packet_thread.taskq);
	down(&packet_thread.control_sem);

	/* success! */
	return 0;
}


/* -----------------------------------------------------------------------------
 * clean up module, exit. This function:
 *     resets netif_rx,
 *     stops dump thread,
 *     brings down all used interfaces.
 * -------------------------------------------------------------------------- */
void ticket_dump_exit(void) {
	int idx;
	struct sk_buff *skb;

	printk(KERN_INFO "Unloading TICKET module... resetting netif_rx.\n");
	netif_rx = old_netif_rx;
#if LINUX_VERSION_CODE>=0x020420
	netif_receive_skb = old_netif_receive;
#endif

	/* --------------------------------------------------------------------------------
	 * kill the kernel thread. must be _AFTER_ reset of netif_rx;
	 * Otherwise we could get more packets which would be messy.
	 * -------------------------------------------------------------------------------- */
	if (packet_thread.thread != NULL) {
		lock_kernel(); /* get BKL, will muck with process data. */
		init_MUTEX_LOCKED(&packet_thread.control_sem); /* We'll wait for thread to stop cleanly.. */
		packet_thread.die = 1; /* tell thread to die... */
		mb(); /* Make sure flags visible on all CPUs. */
		up(&packet_thread.packet_wait_sem); /* Wake thread from packet wait, let it die cleanly */
		down(&packet_thread.control_sem); /* Block till thread function completes cleanup */
		unlock_kernel(); /* release the BKL */
		kill_proc(2, SIGCHLD, 1); /* Now thread is a zombie; Make keventd clean it up. */
	}

	/* --------------------------------------------------------------------------------
	 * now close all devices we opened, require netlink lock
	 * -------------------------------------------------------------------------------- */
	rtnl_lock();
	for (idx=0; idx<ETHERNET_IF_COUNT && ew_eth[idx]!=NULL; idx++) {
		// Only close if that's how it was before. Otherwise, it was open. Don't touch.
		if (ew_eth_oldstatus[idx]==NET_DEVICE_CLOSED) {
			printk(KERN_INFO "Closing %s.\n", ew_eth[idx]->name);
			dev_close(ew_eth[idx]);
		}
		dev_put(ew_eth[idx]);
		ew_eth[idx] = NULL;
	}
	rtnl_unlock();

	// --------------------------------------------------------------------------------
	// clean up rest of memory
	//
	// Remember to free anything in a list (allocates memory per item)
	//    (we don't have any ATM, but may later)
	// --------------------------------------------------------------------------------

	// free output skb if we have one allocated
	if (output_skb!=NULL) {
		printk("Throwing away %u processed packets; incomplete output packet.\n", (*((unsigned *)(&(output_skb->cb[8])))));
		kfree_skb(output_skb);
		output_skb=NULL;
	}

	// free mapping, if we have one allocated
	if (map_addrs!=NULL) {
		free_mapping(map_addrs);
		map_addrs=NULL;
	}

	// Free any packets we haven't gotten around to processing.
	if (skb_queue_len(&ticket_skbs)>0) {
		printk("Throwing away %u unprocessed packets; didn't get to them.\n", skb_queue_len(&ticket_skbs));
		do {
			if ((skb = skb_dequeue(&ticket_skbs))!=NULL) {
				kfree_skb(skb);
			}
		} while (skb_queue_len(&ticket_skbs)>0);
	}
}


/* -----------------------------------------------------------------------------
 * Other stuff; see linux/module.h for more information, or `man modinfo`
 * -------------------------------------------------------------------------- */
#ifdef MODULE
module_init(ticket_dump_init);
module_exit(ticket_dump_exit);

MODULE_AUTHOR("Eric Weigle <ehw@lanl.gov>");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("TICKET kernel code");
MODULE_PARM(pass_through, "i");
MODULE_PARM_DESC(pass_through, "(boolean) Pass packets through to original netif_rx when done?");
#endif

