/*
# TICKET - Traffic Information Collection Kernel with Exact Timing
#
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory for the Regents of the
# University of California.
# 
# Version:	2.0.0-alpha	2003/02/20
# 
# Authors: Eric Weigle, ehw@lanl.gov or radiant-software@lanl.gov
# 
# History:
# 
#   2002-12-24 1.0.0-a  initial release
#   2003-02-20 2.0.0-a  first moderately usable release
# 
# License:
# 
# Copyright 2002. The Regents of the University of California.
# 
# This material was produced under U.S. Government contract
# W-7405-ENG-36 for Los Alamos National Laboratory, which is operated
# by the University of California for the U.S. Department of Energy.
# The Government is granted for itself and other acting on its behalf
# a paid-up, non-exclusive, irrevocable worldwide license in the
# material to reproduce, prepare derivative works, and perform
# publicly and display publicly. Beginning five (5) years after
# November 6, 2002, subject to additional five-year worldwide
# renewals, the Government is granted for itself and others acting on
# its behalf a paid-up, nonexclusive, irrevocable worldwide license
# in this material to reproduce, prepare derivative works, distribute
# copies to the public, perform publicly and display publicly, and to
# permit others to do so. NEITHER THE UNITED STATES NOR THE UNITED
# STATES DEPARTMENT OF ENERGY, NOR THE UNIVERSITY OF CALIFORNIA, NOR
# ANY OF THEIR EMPLOYEES, MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR
# ASSUMES ANY LEGAL LIABLITY OR RESPONSIBILITY FOR THE ACCURACY,
# COMPLETENESS, OR USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT,
# OR PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE
# PRIVATELY OWNED RIGHTS.
# 
# Additionally, this program is free software; you can distribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 of
# the License, or any later version. Accordingly, this program is
# distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
# License for more details. You should have received a copy of the
# GNU General Public License along with this program; if not, write
# to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
*/


// #############################################################################
// Header file for administrative program.
// #############################################################################

#ifndef TICKET_ADMIN_H
#define TICKET_ADMIN_H

// ============================================================================
// Included files, muck with some lower-level stuff so there are a few...
// ============================================================================
#include <arpa/inet.h>   // inet_ntoa()
#include <assert.h>      // assert()
#include <fcntl.h>       // open()
#include <net/if.h>      // ioctl dependencies
#include <netdb.h>       // gethostbyname(), hostent
#include <netinet/in.h>  // sockaddr_in
#include <signal.h>      // sigaction
#include <stdio.h>       // printf()
#include <sys/ioctl.h>   // ioctl, raw interface info
#include <sys/socket.h>  // socket()
#include <sys/types.h>   // open()
#include <unistd.h>      // close()
#include <string.h>      // strlen, parse routines
#include <stdlib.h>      // for system.
#include <errno.h>       // errno.

#include <pwd.h> // getpwnam
#include <grp.h> // initgroups

// Code to get in the right socket() headers.
#include <features.h>    // for the glibc version number
#if __GLIBC__ >= 2 && __GLIBC_MINOR >= 1
#include <netpacket/packet.h>
#include <net/ethernet.h>     // the L2 protocols
#else
#include <asm/types.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>   // The L2 protocols
#endif


// ============================================================================
// #defines
// ============================================================================
// This is how long the 'admin' packet should be.
// 16 bytes ethernet header
// 32 bytes packet data (only 4 currently used)
#define DIE_LEN 32+16
// This has to match what's in the TICKET kernel code.
#define ETH_PROTOCOL 0x9999

// This has to match what's in the TICKET kernel code too.
#define MODE_ERROR 0
#define MODE_KILL  1
#define MODE_GET   2
#define MODE_SET   3

// allocate the maximum ethernet size packet.
#ifndef ADMIN_PACKET_SIZE
#define ADMIN_PACKET_SIZE 1514
#endif

// ethernet header length.
#ifndef ETH_HDR_LEN
#define ETH_HDR_LEN 14
#endif

// minimum packet size.
#ifndef MIN_ETH_LEN
#define MIN_ETH_LEN 64
#endif


// ============================================================================
// FUNCTION DECLARATIONS
// ============================================================================
void makekill();
int  sendkill();
void usage(char *error);
int  root_check(int argc, char *argv[]);

#endif

