/*
# TICKET - Traffic Information Collection Kernel with Exact Timing
#
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory for the Regents of the
# University of California.
# 
# Version:	2.0.0-alpha	2003/02/20
# 
# Authors: Eric Weigle, ehw@lanl.gov or radiant-software@lanl.gov
# 
# History:
# 
#   2002-12-24 1.0.0-a  initial release
#   2003-02-20 2.0.0-a  first moderately usable release
# 
# License:
# 
# Copyright 2002. The Regents of the University of California.
# 
# This material was produced under U.S. Government contract
# W-7405-ENG-36 for Los Alamos National Laboratory, which is operated
# by the University of California for the U.S. Department of Energy.
# The Government is granted for itself and other acting on its behalf
# a paid-up, non-exclusive, irrevocable worldwide license in the
# material to reproduce, prepare derivative works, and perform
# publicly and display publicly. Beginning five (5) years after
# November 6, 2002, subject to additional five-year worldwide
# renewals, the Government is granted for itself and others acting on
# its behalf a paid-up, nonexclusive, irrevocable worldwide license
# in this material to reproduce, prepare derivative works, distribute
# copies to the public, perform publicly and display publicly, and to
# permit others to do so. NEITHER THE UNITED STATES NOR THE UNITED
# STATES DEPARTMENT OF ENERGY, NOR THE UNIVERSITY OF CALIFORNIA, NOR
# ANY OF THEIR EMPLOYEES, MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR
# ASSUMES ANY LEGAL LIABLITY OR RESPONSIBILITY FOR THE ACCURACY,
# COMPLETENESS, OR USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT,
# OR PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE
# PRIVATELY OWNED RIGHTS.
# 
# Additionally, this program is free software; you can distribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 of
# the License, or any later version. Accordingly, this program is
# distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
# License for more details. You should have received a copy of the
# GNU General Public License along with this program; if not, write
# to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
*/

// #############################################################################
// This file takes care of differences between user and kernel level code.
// In particular, code compiled at both user and kernel level has several
// somewhat arbitrary but simple changes, such as printf<-->printk, etc.
// We account for these with some #defines.
// #############################################################################

#ifndef TICKET_USERKERNEL_H
#define TICKET_USERKERNEL_H

#ifdef __KERNEL__
// =============================================================================
// if at kernel level, then set up kernel stuff.
// =============================================================================

#define ALLOC_FUNCTION kmalloc
#define ALLOC_PRIORITY , GFP_KERNEL
#define FREE_FUNCTION kfree
#define PRINT_FUNCTION printk
#define PRIORITY_NORMAL KERN_NOTICE
#define PRIORITY_ERROR KERN_EMERG
void ew_panic(const char *msg);

#else
// =============================================================================
// If at user level, then set up user stuff.
// =============================================================================

#define ALLOC_FUNCTION malloc
#define ALLOC_PRIORITY /* none for user level, priority for kernel */
#define FREE_FUNCTION free
/* 20030124-changed all to printf; having problems sharing with kernel code
 * because w/o a newline, two printk(KEN_NOTICE, "Foo"); show up as "Foo<5>Foo"
 * not "FooFoo" like we want. Other approaches get painful fast. */
#define PRINT_FUNCTION printf
#define PRIORITY_NORMAL
#define PRIORITY_ERROR
// This is an actual function in the kernel, here just print and die.
#define ew_panic(x) fprintf(stderr, x); exit(1)

#endif

#endif
