/*
# TICKET - Traffic Information Collection Kernel with Exact Timing
#
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory for the Regents of the
# University of California.
# 
# Version:	2.0.0-alpha	2003/02/20
# 
# Authors: Eric Weigle, ehw@lanl.gov or radiant-software@lanl.gov
# 
# History:
# 
#   2002-12-24 1.0.0-a  initial release
#   2003-02-20 2.0.0-a  first moderately usable release
# 
# License:
# 
# Copyright 2002. The Regents of the University of California.
# 
# This material was produced under U.S. Government contract
# W-7405-ENG-36 for Los Alamos National Laboratory, which is operated
# by the University of California for the U.S. Department of Energy.
# The Government is granted for itself and other acting on its behalf
# a paid-up, non-exclusive, irrevocable worldwide license in the
# material to reproduce, prepare derivative works, and perform
# publicly and display publicly. Beginning five (5) years after
# November 6, 2002, subject to additional five-year worldwide
# renewals, the Government is granted for itself and others acting on
# its behalf a paid-up, nonexclusive, irrevocable worldwide license
# in this material to reproduce, prepare derivative works, distribute
# copies to the public, perform publicly and display publicly, and to
# permit others to do so. NEITHER THE UNITED STATES NOR THE UNITED
# STATES DEPARTMENT OF ENERGY, NOR THE UNIVERSITY OF CALIFORNIA, NOR
# ANY OF THEIR EMPLOYEES, MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR
# ASSUMES ANY LEGAL LIABLITY OR RESPONSIBILITY FOR THE ACCURACY,
# COMPLETENESS, OR USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT,
# OR PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE
# PRIVATELY OWNED RIGHTS.
# 
# Additionally, this program is free software; you can distribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 of
# the License, or any later version. Accordingly, this program is
# distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
# License for more details. You should have received a copy of the
# GNU General Public License along with this program; if not, write
# to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
*/

// #############################################################################
// PRINTING FUNCTIONS... these are some formats that may be useful.
//     Pick one that makes sense to you, or modify to taste.
// #############################################################################
#include <time.h> // localtime, struct timespec
#include "read_anon.h"
#include "read_common.h"
#include "print.h"
#include "../common/addresses.h"

// FILE-GLOBALS
char pbuf1[32], pbuf2[32];

// // =============================================================================
// // The meta-print function. Just an iterator over the others, to print all
// // data in a packet...
// // =============================================================================
// void print_dump_packet(FILE *file, struct dump_packet *p, void (*print_fn)(FILE *file, struct collected_headers_ticket *data)) {
// 	unsigned int idx;
// 
// 	fprintf(file, "(%08X) p->seqno: %lld\n", (unsigned int)&(p->seqno), p->seqno);
// 
// 	// these headers are those from the dump machine
// 	for (idx=0; idx<DUMPS_PER_PACKET; idx++) {
// 		fprintf(file, "data[%d]\t", idx);
// 		(*print_fn)(file, &(p->data[idx]));
// 	}
// }


// =============================================================================
// 'dumb' format; print out MEMORY offsets and HEX values
// =============================================================================
void print_data_dumb(FILE *file, const struct collected_headers_ticket *data) {
	fprintf(file, "(%08X) data:\n", (unsigned int)data);
	fprintf(file, "(%08X) timestamp: %lld\n" , (unsigned int)&(data->timestamp)        , data->timestamp);
	fprintf(file, "(%08X) length: %04X\n"    , (unsigned int)&(data->length)           , data->length);
	fprintf(file, "(%08X) mac_src: %s\n"     , (unsigned int)data->mac_src             , mac2str(data->mac_src, pbuf1));
	fprintf(file, "(%08X) mac_dst: %s\n"     , (unsigned int)data->mac_dst             , mac2str(data->mac_dst, pbuf1));
	fprintf(file, "(%08X) toslen: %04X\n"    , (unsigned int)&(data->toslen)           , data->toslen);
	fprintf(file, "(%08X) ip_src: %08X\n"    , (unsigned int)&(data->ip_src)           , data->ip_src);
	fprintf(file, "(%08X) ip_dst: %08X\n"    , (unsigned int)&(data->ip_dst)           , data->ip_dst);
	fprintf(file, "(%08X) ip_len: %04X\n"    , (unsigned int)&(data->ip_len)           , data->ip_len);
	fprintf(file, "(%08X) ip_hdr_len: %02X\n", (unsigned int)&(data->ip_hdr_len)       , data->ip_hdr_len);
	fprintf(file, "(%08X) protocol: %02X\n"  , (unsigned int)&(data->protocol)         , data->protocol);
	fprintf(file, "(%08X) src_port: %04X\n"  , (unsigned int)&(data->src_port)         , data->src_port);
	fprintf(file, "(%08X) dst_port: %04X\n"  , (unsigned int)&(data->dst_port)         , data->dst_port);
	fprintf(file, "(%08X) udp_len: %04X\n"   , (unsigned int)&(data->upper_len).udp_len, data->upper_len.udp_len);
}


// =============================================================================
// 'smart' format; print out INDEXED offsets and INTEGRAL values
// =============================================================================
void print_data_smart(FILE *file, const struct collected_headers_ticket *data) {
	// These are the actual data points.
	fprintf(file, "  timestamp:  %lld\n",   data->timestamp);
	fprintf(file, "  length:     %d\n",     data->length);
	fprintf(file, "  mac_src[6]: %s\n",     mac2str(data->mac_src, pbuf1));
	fprintf(file, "  mac_dst[6]: %s\n",     mac2str(data->mac_dst, pbuf2));
	fprintf(file, "  toslen:     0x%04X\n", data->toslen);
	fprintf(file, "  ip_src:     %s\n",     ip2str_hbo(data->ip_src, pbuf1));
	fprintf(file, "  ip_dst:     %s\n",     ip2str_hbo(data->ip_dst, pbuf2));
	fprintf(file, "  ip_len:     %d\n",     data->ip_len);
	fprintf(file, "  ip_hdr_len: %d\n",     data->ip_hdr_len);
	fprintf(file, "  protocol:   %d\n",     data->protocol);
	fprintf(file, "  src_port:   %d\n",     data->src_port);
	fprintf(file, "  dst_port:   %d\n",     data->dst_port);
	// This isn't that 'smart', TCP: hdr_len is char, UDP: length is short... see traf.h
	fprintf(file, "  upper_len:  %d\n",     data->upper_len.udp_len);
}


// =============================================================================
// 'minimal' format; prints only:
//     addresses, types of service, protocol, and ports.
// =============================================================================
void print_data_minimal(FILE *file, const struct collected_headers_ticket *data) {
	// These are the actual data points. Print on one line...
	fprintf(file, "%04X\t%d", data->toslen, data->protocol);
	fprintf(file, "\t%s\t%s", mac2str(data->mac_src, pbuf1), mac2str(data->mac_dst, pbuf2));
	if (data->toslen == 0x0800) {
		fprintf(file, "\t%s", ip2str_hbo(data->ip_src, pbuf1));
		if (data->protocol==TCP_HDR_PROTOCOL || data->protocol==UDP_HDR_PROTOCOL) {
			fprintf(file, ":%d", data->src_port);
		}
		fprintf(file, "\t%s", ip2str_hbo(data->ip_dst, pbuf2));
		if (data->protocol==TCP_HDR_PROTOCOL || data->protocol==UDP_HDR_PROTOCOL) {
			fprintf(file, ":%d", data->dst_port);
		}
	}
	fprintf(file, "\n");
}


// =============================================================================
// 'tcpdump' format; tries to somewhat mimic output of 'tcpdump' program
// =============================================================================
void print_data_tcpdump(FILE *file, const struct collected_headers_ticket *data) {
	// time first.
	fprintf(file, "%lld\t",  data->timestamp);

	// all the data we can be sure of having... MAC level stuff.
	fprintf(file, "%s > %s\t", mac2str(data->mac_src, pbuf1),    mac2str(data->mac_dst, pbuf2));
	fprintf(file, "%4d\t", data->length);
	fprintf(file, "%04X\t|\t", data->toslen);

	// IP information, if it exists.
	if (data->toslen == 0x0800) {
		fprintf(file, "%s -> %s\t", ip2str_hbo(data->ip_src, pbuf1), ip2str_hbo(data->ip_dst, pbuf2));
		fprintf(file, "%4d\t%d\t%d\t|\t", data->ip_len, data->ip_hdr_len, data->protocol);
		// fprintf(file, "%d\t", hdr.ip_len); // always hdr.length - 14, unless a fragment.
		// fprintf(file, "%d\t", hdr.ip_hdr_len); // nearly always 20.

		// finally, upper-level stuff (if it exists).
		if (data->protocol == UDP_HDR_PROTOCOL ||data->protocol == TCP_HDR_PROTOCOL) {
			// UDP header included in this length measure. It's a constant eight bytes.
			// udp_len should be length - 14 (ethernet) - 20ish (ip)
			// TCP header length is what this field holds after processing in ntoh_header.
			fprintf(file, "(%s)\t", (data->protocol==UDP_HDR_PROTOCOL ? "UDP" : "TCP"));
			fprintf(file, "%d => %d\t%d", data->src_port, data->dst_port, data->upper_len.udp_len);
		}
	}
	fprintf(file, "\n");
}


// =============================================================================
// Print out anonymized data->
// =============================================================================
void print_data_processed(FILE *file, const struct processed_headers *data) {
	struct tm *local_time;

	// muck with time first. Don't use tm_wday, tm_yday, tm_isdst.
	local_time = localtime(&(data->timestamp.tv_sec));
	fprintf(file, "%04d%02d%02d %02d:%02d:%02d.%09lu\t",
			1900+local_time->tm_year, 1+local_time->tm_mon, local_time->tm_mday,
			local_time->tm_hour, local_time->tm_min, local_time->tm_sec, (unsigned long)data->timestamp.tv_nsec);

	// all the data we can be sure of having... MAC level stuff.
	fprintf(file, "%d > %d\t", data->mac_src, data->mac_dst);
	fprintf(file, "%d\t", data->length);
	fprintf(file, "%04X\t|\t", data->toslen);

	// IP information, if it exists.
	if (data->toslen == 0x0800) {
		fprintf(file, "%d -> %d\t", data->ip_src, data->ip_dst);
		fprintf(file, "%4d\t%d\t%d\t|\t", data->ip_len, data->ip_hdr_len, data->protocol);

		// finally, upper-level stuff (if it exists).
		if (data->protocol==TCP_HDR_PROTOCOL || data->protocol==UDP_HDR_PROTOCOL) {
			fprintf(file, "(%s)\t", (data->protocol==UDP_HDR_PROTOCOL ? "UDP" : "TCP"));
			fprintf(file, "%d => %d\t%d", data->src_port, data->dst_port, data->upper_len);
		}
	}
	fprintf(file, "\n");
}

